# /=====================================================================\ #
# |  Version                                                            | #
# | Record LaTeXML Version, Revision...                                 | #
# |=====================================================================| #
# | Part of LaTeXML:                                                    | #
# |  Public domain software, produced as part of work done by the       | #
# |  United States Government & not subject to copyright in the US.     | #
# |---------------------------------------------------------------------| #
# | Bruce Miller <bruce.miller@nist.gov>                        #_#     | #
# | http://dlmf.nist.gov/LaTeXML/                              (o o)    | #
# \=========================================================ooo==U==ooo=/ #

package LaTeXML::Version;
use strict;
use warnings;
use FindBin;

#======================================================================
# This puts versioning in one place so that all components of LaTeXML
# can easily see it with only a minimal import.
# However, we still must contrive that $VERSION is visible
# DIRECTLY in LaTeXML.pm, otherwise MakeMaker won't see it
# and modules that depend on LaTeXML won't see the version number!
# This formulation:
###   $LaTeXML::VERSION = do { use LaTeXML::Version; $LaTeXML::Version::VERSION; };
# seems to work around ExtUtils::MakeMaker's get_version.
# Keep this as $LaTeXML::Version::VERSION, since get_version clears $VERSION!!!
#======================================================================

use vars qw($VERSION);

# This is the main version of LaTeXML being claimed.
use version; our $VERSION = version->declare("0.8.0");

$LaTeXML::VERSION = $VERSION;   # "export" it...

# This is the git/svn/whatever revision; it should be filled in by make.
$LaTeXML::REVISION = "";

# Derived, more informative version numbers
$LaTeXML::FULLVERSION = "LaTeXML version $LaTeXML::VERSION"
  . ($LaTeXML::REVISION ? "; revision $LaTeXML::REVISION" : '');

# Handy identifier string for any executable.
$LaTeXML::IDENTITY = "$FindBin::Script ($LaTeXML::FULLVERSION)";

1;

